#encoding=utf8

__version__ = '0.1.5'
__all__ = [
    'MLBase',
    'PLA',
    'PLA.BinaryClassifier',
    'PLA.MultiClassifier',
    'PocketPLA',
    'PocketPLA.BinaryClassifier',
    'PocketPLA.MultiClassifier',
    'LinearRegression',
    'LinearRegression.LinearRegression',
    'LinearRegression.BinaryClassifier',
    'LinearRegression.MultiClassifier',
    'LinearRegression.Accelerator',
    'LogisticRegression.LogisticRegression',
    'LogisticRegression.BinaryClassifier',
    'LogisticRegression.MultiClassifier',
    'Utility',
    'Utility.DatasetLoader',
    'Utility.CrossValidator',
    'Utility.Serializer'
]
